@@ -1,8 +1,8 @@
 /*
- * dselect - Debian GNU/Linux package maintenance user interface
+ * dselect - Debian package maintenance user interface
  * pkgcmds.cc - package list keyboard commands
  *
- * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -126,7 +126,13 @@ void packagelist::setwant(pkginfo::pkgwant nwarg) {
   movecursorafter(bot);
 }
 
-void packagelist::kd_select()   { setwant(pkginfo::want_install);   }
+int manual_install = 0;
+
+void packagelist::kd_select()   {
+	manual_install = 1;
+	setwant(pkginfo::want_install);
+	manual_install = 0;
+}
 void packagelist::kd_hold()     { setwant(pkginfo::want_hold);      }
 void packagelist::kd_deselect() { setwant(pkginfo::want_deinstall); }
 void packagelist::kd_unhold()   { setwant(pkginfo::want_sentinel);  }
