@@ -2,7 +2,7 @@
  * libdpkg - Debian packaging suite library routines
  * dpkg-db.h - Low level package database routines (hash tables, etc.)
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -169,13 +169,19 @@ int informative(struct pkginfo *pkg, struct pkginfoperfile *info) {
   return 0;
 }
 
-struct pkginfo *findpackage(const char *name) {
+struct pkginfo *findpackage(const char *inname) {
   struct pkginfo **pointerp, *newpkg;
+  char *name = strdup(inname), *p;
 
+  if (name == NULL)
+    ohshite(_("couldn't allocate memory for strdup in findpackage(%s)"),inname);
+  p= name;
+  while(*p) { *p= tolower(*p); p++; }
+  
   pointerp= bins + (hash(name) & (BINS-1));
   while (*pointerp && strcasecmp((*pointerp)->name,name))
     pointerp= &(*pointerp)->next;
-  if (*pointerp) return *pointerp;
+  if (*pointerp) { free(name); return *pointerp; }
 
   newpkg= nfmalloc(sizeof(struct pkginfo));
   blankpackage(newpkg);
@@ -184,6 +190,7 @@ struct pkginfo *findpackage(const char *name) {
   *pointerp= newpkg;
   npackages++;
 
+  free(name);
   return newpkg;
 }
 
