@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * dbmodify.c - routines for managing dpkg database updates
  *
- * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
  * Copyright (C) 2001 Wichert Akkerman <wichert@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -53,7 +53,7 @@ static int ulist_select(const struct dirent *de) {
   const char *p;
   int l;
   for (p= de->d_name, l=0; *p; p++, l++)
-    if (!isdigit(*p)) return 0;
+    if (!cisdigit(*p)) return 0;
   if (l > IMPORTANTMAXLEN)
     ohshit(_("updates directory contains file `%.250s' whose name is too long "
            "(length=%d, max=%d)"), de->d_name, l, IMPORTANTMAXLEN);
@@ -101,7 +101,7 @@ static void cleanupdates(void) {
 }
 
 static void createimptmp(void) {
-  int i, f;
+  int i;
   
   onerr_abort++;
   
@@ -119,13 +119,15 @@ static void createimptmp(void) {
   onerr_abort--;
 }
 
+const struct fni { const char *suffix; char **store; } fnis[]= {
+  {   STATUSFILE,                 &statusfile         },
+  {   AVAILFILE,                  &availablefile      },
+  {   UPDATESDIR IMPORTANTTMP,    &importanttmpfile   },
+  {   NULL, NULL                                      }
+};
+
 enum modstatdb_rw modstatdb_init(const char *adir, enum modstatdb_rw readwritereq) {
-  static const struct fni { const char *suffix; char **store; } fnis[]= {
-    {   STATUSFILE,                 &statusfile         },
-    {   AVAILFILE,                  &availablefile      },
-    {   UPDATESDIR IMPORTANTTMP,    &importanttmpfile   },
-    {   NULL, NULL                                      }
-  }, *fnip;
+  const struct fni *fnip;
   
   admindir= adir;
 
@@ -203,6 +205,7 @@ static void checkpoint(void) {
 }
 
 void modstatdb_shutdown(void) {
+  const struct fni *fnip;
   switch (cstatus) {
   case msdbrw_write:
     checkpoint();
@@ -218,6 +221,10 @@ void modstatdb_shutdown(void) {
     break;
   }
 
+  for (fnip=fnis; fnip->suffix; fnip++) {
+    free(*fnip->store);
+    *fnip->store= NULL;
+  }
   free(updatefnbuf);
 }
 
