@@ -2,7 +2,8 @@
  * libdpkg - Debian packaging suite library routines
  * fields.c - parsing of all the different fields, when reading in
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright 2001 Wichert Akkerman
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -71,12 +72,10 @@ void f_name(struct pkginfo *pigp, struct pkginfoperfile *pifp, enum parsedbflags
   const char *e;
   if ((e= illegal_packagename(value,NULL)) != NULL)
     parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("invalid package name (%.250s)"),e);
-  pigp->name= nfstrsave(value);
-  findpackage(value); /* We discard the value from findpackage, but calling it
-                       * forces an entry in the hash table to be made if it isn't
-                       * already.  This is so that we don't reorder the file
-                       * unnecessarily (doing so is bad for debugging).
-                       */
+  pigp->name= findpackage(value)->name;
+  /* We use the new name, as findpackage() may have
+     done a tolower for us.
+   */
 }
 
 void f_filecharf(struct pkginfo *pigp, struct pkginfoperfile *pifp,
@@ -131,7 +130,7 @@ void f_boolean(struct pkginfo *pigp, struct pkginfoperfile *pifp,
                const char *filename, int lno, FILE *warnto, int *warncount,
                const char *value, const struct fieldinfo *fip) {
   pifp->essential=
-    *value ? convert_string(filename,lno,_("yes/no in `essential' field"), -1,
+    *value ? convert_string(filename,lno,_("yes/no in boolean field"), -1,
                             warnto,warncount,pigp,
                             value,booleaninfos,NULL)
            : 0;
@@ -376,6 +375,7 @@ void f_dependency(struct pkginfo *pigp, struct pkginfoperfile *pifp,
           if (isspace(*p)) break;
           p++;
         }
+	versionlength= p - versionstart;
         while (isspace(*p)) p++;
         if (*p == '(') parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                                 _("`%s' field, reference to `%.255s': "
@@ -386,7 +386,6 @@ void f_dependency(struct pkginfo *pigp, struct pkginfoperfile *pifp,
         else if (*p == 0) parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                                    _("`%s' field, reference to `%.255s': "
                                    "version unterminated"),fip->name,depname);
-	versionlength= p - versionstart;
 	if (versionlength >=  versionused) {
 	  versionused= versionlength;
 	  version= realloc(version,versionlength+1);
