@@ -2,7 +2,7 @@
  * libdpkg - Debian packaging suite library routines
  * parsehelp.c - helpful routines for parsing and writing
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -214,7 +214,7 @@ const char *parseversion(struct versionrevision *rversion, const char *string) {
   } else {
     rversion->epoch= 0;
   }
-  rversion->version= nfstrnsave(string,end-string+1);
+  rversion->version= nfstrnsave(string,end-string);
   hyphen= strrchr(rversion->version,'-');
   if (hyphen) *hyphen++= 0;
   rversion->revision= hyphen ? hyphen : "";
@@ -228,9 +228,7 @@ void parsemustfield
  const struct pkginfo *pigp, int warnonly,
  const char **value, const char *what) 
 {
-  static char *empty = NULL;
-  if (!empty)
-    empty= nfstrsave("");
+  static const char *empty = "";
   if (!*value) {
     parseerr(file,filename,lno, warnto,warncount,pigp,warnonly, _("missing %s"),what);
     *value= empty;
