@@ -2,7 +2,7 @@
  * libdpkg - Debian packaging suite library routines
  * vercmp.c - comparison of version numbers
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -32,6 +32,41 @@ int epochsdiffer(const struct versionrevision *a,
   return a->epoch != b->epoch;
 }
 
+/* assume ascii; warning: evaluates x multiple times! */
+#define order(x) ((x) == '~' ? -1 \
+		: cisdigit((x)) ? 0 \
+		: !(x) ? 0 \
+		: cisalpha((x)) ? (x) \
+		: (x) + 256)
+
+static int verrevcmp(const char *val, const char *ref) {
+  if (!val) val= "";
+  if (!ref) ref= "";
+
+  while (*val || *ref) {
+    int first_diff= 0;
+
+    while ( (*val && !cisdigit(*val)) || (*ref && !cisdigit(*ref)) ) {
+      int vc= order(*val), rc= order(*ref);
+      if (vc != rc) return vc - rc;
+      val++; ref++;
+    }
+
+    while ( *val == '0' ) val++;
+    while ( *ref == '0' ) ref++;
+    while (cisdigit(*val) && cisdigit(*ref)) {
+      if (!first_diff) first_diff= *val - *ref;
+      val++; ref++;
+    }
+    if (cisdigit(*val)) return 1;
+    if (cisdigit(*ref)) return -1;
+    if (first_diff) return first_diff;
+  }
+  return 0;
+}
+
+
+#if 0
 static int verrevcmp(const char *ival, const char *iref) {
   static char empty[] = "";
   int vc, rc;
@@ -64,6 +99,7 @@ static int verrevcmp(const char *ival, const char *iref) {
     if (!*ref) return +1;
   }
 }
+#endif
 
 int versioncompare(const struct versionrevision *version,
                    const struct versionrevision *refversion) {
