@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * archives.c - actions that process archive files, mainly unpack
  *
- * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
  * Copyright (C) 2000 Wichert Akkerman <wakkerma@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -131,7 +131,7 @@ static struct obstack tar_obs;
 static int tarobs_init= 0;
 
 /* ensure the obstack is properly initialized */
-static void ensureobstackinit() {
+static void ensureobstackinit(void) {
 
   if (!tarobs_init) {
     obstack_init(&tar_obs);
@@ -140,7 +140,7 @@ static void ensureobstackinit() {
 }
 
 /* destroy the obstack */
-static void destroyobstack() {
+static void destroyobstack(void) {
   if (tarobs_init) {
     obstack_free(&tar_obs, 0);
     tarobs_init= 0;
@@ -653,8 +653,7 @@ static int try_remove_can(struct deppossi *pdep,
       }
     }
     pdep->up->up->clientdata->istobe= itb_deconfigure;
-    ensureobstackinit();
-    newdeconf= obstack_alloc(&tar_obs, sizeof(struct packageinlist));
+    newdeconf= malloc(sizeof(struct packageinlist));
     newdeconf->next= deconfigure;
     newdeconf->pkg= pdep->up->up;
     deconfigure= newdeconf;
@@ -937,7 +936,7 @@ int wanttoinstall(struct pkginfo *pkg, const struct versionrevision *ver, int sa
   enum versiondisplayepochwhen needepochs;
   int r;
 
-  if (pkg->want != want_install) {
+  if (pkg->want != want_install && pkg->want != want_hold) {
     if (f_alsoselect) {
       if (saywhy) {
    printf(_("Selecting previously deselected package %s.\n"),pkg->name);
