@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * enquiry.c - status enquiry and listing options
  *
- * Copyright (C) 1995,1996 Ian Jackson <ijackson@gnu.ai.mit.edu>
+ * Copyright (C) 1995,1996 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -60,112 +60,6 @@ static void limiteddescription(struct pkginfo *pkg, int maxl,
   *pdesc_r=pdesc; *l_r=l;
 }
 
-static int getwidth() {
-  int fd;
-  int res;
-  struct winsize ws;
-  const char* columns;
-
-  if ((columns=getenv("COLUMNS")) && ((res=atoi(columns))>0))
-    ws.ws_col=res;
-  else if (!isatty(1))
-    ws.ws_col=80;
-  else {
-    if ((fd=open("/dev/tty",O_RDONLY))!=-1) {
-      if (ioctl(fd, TIOCGWINSZ, &ws)==-1)
-        ws.ws_col=80;
-      close(fd);
-    }
-  }
-  return ws.ws_col;
-}
-
-static void list1package(struct pkginfo *pkg, int *head) {
-  int l,w;
-  static int nw,vw,dw;
-  const char *pdesc;
-  static char format[80]   = "";
-    
-  if (format[0]==0) {
-    w=getwidth()-80;	/* get spare width */
-    if (w<0) w=0;		/* lets not try to deal with terminals that are too small */
-    w>>=2;		/* halve that so we can add that to the both the name and description */
-    nw=(14+w);		/* name width */
-    vw=(14+w);		/* version width */
-    dw=(44+(2*w));	/* description width */
-    sprintf(format,"%%c%%c%%c %%-%d.%ds %%-%d.%ds %%.*s\n", nw, nw, vw, vw);
-  }
-
-  if (!*head) {
-    fputs(_("\
-Desired=Unknown/Install/Remove/Purge/Hold\n\
-| Status=Not/Installed/Config-files/Unpacked/Failed-config/Half-installed\n\
-|/ Err?=(none)/Hold/Reinst-required/X=both-problems (Status,Err: uppercase=bad)\n"), stdout);
-    printf(format,'|','|','/', _("Name"), _("Version"), 40, _("Description"));
-    printf("+++-");					/* status */
-    for (l=0;l<nw;l++) printf("="); printf("-");	/* packagename */
-    for (l=0;l<vw;l++) printf("="); printf("-");	/* version */
-    for (l=0;l<dw;l++) printf("="); 			/* description */
-    printf("\n");
-    *head= 1;
-  }
-  if (!pkg->installed.valid) blankpackageperfile(&pkg->installed);
-  limiteddescription(pkg,dw,&pdesc,&l);
-  printf(format,
-         "uihrp"[pkg->want],
-         "nUFiHc"[pkg->status],
-         " R?#"[pkg->eflag],
-         pkg->name,
-         versiondescribe(&pkg->installed.version,vdew_never),
-         l, pdesc);
-}         
-
-void listpackages(const char *const *argv) {
-  struct pkgiterator *it;
-  struct pkginfo *pkg;
-  struct pkginfo **pkgl;
-  const char *thisarg;
-  int np, i, head, found;
-
-  modstatdb_init(admindir,msdbrw_readonly);
-
-  np= countpackages();
-  pkgl= m_malloc(sizeof(struct pkginfo*)*np);
-  it= iterpkgstart(); i=0;
-  while ((pkg= iterpkgnext(it))) {
-    assert(i<np);
-    pkgl[i++]= pkg;
-  }
-  iterpkgend(it);
-  assert(i==np);
-
-  qsort(pkgl,np,sizeof(struct pkginfo*),pkglistqsortcmp);
-  head=0;
-  
-  if (!*argv) {
-    for (i=0; i<np; i++) {
-      pkg= pkgl[i];
-      if (pkg->status == stat_notinstalled) continue;
-      list1package(pkg,&head);
-    }
-  } else {
-    while ((thisarg= *argv++)) {
-      found= 0;
-      for (i=0; i<np; i++) {
-        pkg= pkgl[i];
-        if (fnmatch(thisarg,pkg->name,0)) continue;
-        list1package(pkg,&head); found++;
-      }
-      if (!found) {
-        fprintf(stderr,_("No packages found matching %s.\n"),thisarg);
-	nerrs++;
-      }
-    }
-  }
-  if (ferror(stdout)) werr("stdout");
-  if (ferror(stderr)) werr("stderr");  
-}
-
 struct badstatinfo {
   int (*yesno)(struct pkginfo*, const struct badstatinfo *bsi);
   int val;
@@ -343,174 +237,6 @@ void unpackchk(const char *const *argv) {
   if (ferror(stdout)) werr("stdout");
 }
 
-static int searchoutput(struct filenamenode *namenode) {
-  int found, i;
-  struct filepackages *packageslump;
-
-  if (namenode->divert) {
-    for (i=0; i<2; i++) {
-      if (namenode->divert->pkg) printf(_("diversion by %s"),namenode->divert->pkg->name);
-      else printf(_("local diversion"));
-      printf(" %s: %s\n", i ? _("to") : _("from"),
-             i ?
-             (namenode->divert->useinstead
-              ? namenode->divert->useinstead->name
-              : namenode->name)
-             :
-             (namenode->divert->camefrom
-              ? namenode->divert->camefrom->name
-              : namenode->name));
-    }
-  }
-  found= 0;
-  for (packageslump= namenode->packages;
-       packageslump;
-       packageslump= packageslump->more) {
-    for (i=0; i < PERFILEPACKAGESLUMP && packageslump->pkgs[i]; i++) {
-      if (found) fputs(", ",stdout);
-      fputs(packageslump->pkgs[i]->name,stdout);
-      found++;
-    }
-  }
-  if (found) printf(": %s\n",namenode->name);
-  return found + (namenode->divert ? 1 : 0);
-}
-
-void searchfiles(const char *const *argv) {
-  struct filenamenode *namenode;
-  struct fileiterator *it;
-  const char *thisarg;
-  int found;
-  static struct varbuf vb;
-  
-  if (!*argv)
-    badusage(_("--search needs at least one file name pattern argument"));
-
-  modstatdb_init(admindir,msdbrw_readonly|msdbrw_noavail);
-  ensure_allinstfiles_available_quiet();
-  ensure_diversions();
-
-  while ((thisarg= *argv++) != 0) {
-    found= 0;
-    if (!strchr("*[?/",*thisarg)) {
-      varbufreset(&vb);
-      varbufaddc(&vb,'*');
-      varbufaddstr(&vb,thisarg);
-      varbufaddc(&vb,'*');
-      varbufaddc(&vb,0);
-      thisarg= vb.buf;
-    }
-    if (strcspn(thisarg,"*[?\\") == strlen(thisarg)) {
-      namenode= findnamenode(thisarg, 0);
-      found += searchoutput(namenode);
-    } else {
-      it= iterfilestart();
-      while ((namenode= iterfilenext(it)) != 0) {
-        if (fnmatch(thisarg,namenode->name,0)) continue;
-        found+= searchoutput(namenode);
-      }
-      iterfileend(it);
-    }
-    if (!found) {
-      fprintf(stderr,_("dpkg: %s not found.\n"),thisarg);
-      nerrs++;
-      if (ferror(stderr)) werr("stderr");
-    } else {
-      if (ferror(stdout)) werr("stdout");
-    }
-  }
-}
-
-void enqperpackage(const char *const *argv) {
-  int failures;
-  const char *thisarg;
-  struct fileinlist *file;
-  struct pkginfo *pkg;
-  struct filenamenode *namenode;
-  
-  if (!*argv)
-    badusage(_("--%s needs at least one package name argument"), cipaction->olong);
-
-  failures= 0;
-  if (cipaction->arg==act_listfiles)
-    modstatdb_init(admindir,msdbrw_readonly|msdbrw_noavail);
-  else 
-    modstatdb_init(admindir,msdbrw_readonly);
-
-  while ((thisarg= *argv++) != 0) {
-    pkg= findpackage(thisarg);
-
-    switch (cipaction->arg) {
-      
-    case act_status:
-      if (pkg->status == stat_notinstalled &&
-          pkg->priority == pri_unknown &&
-          !(pkg->section && *pkg->section) &&
-          !pkg->files &&
-          pkg->want == want_unknown &&
-          !informative(pkg,&pkg->installed)) {
-        fprintf(stderr,_("Package `%s' is not installed and no info is available.\n"),pkg->name);
-        failures++;
-      } else {
-        writerecord(stdout, "<stdout>", pkg, &pkg->installed);
-      }
-      break;
-
-    case act_printavail:
-      if (!informative(pkg,&pkg->available)) {
-        fprintf(stderr,_("Package `%s' is not available.\n"),pkg->name);
-        failures++;
-      } else {
-        writerecord(stdout, "<stdout>", pkg, &pkg->available);
-      }
-      break;
-      
-    case act_listfiles:
-      switch (pkg->status) {
-      case stat_notinstalled: 
-        fprintf(stderr,_("Package `%s' is not installed.\n"),pkg->name);
-        failures++;
-        break;
-        
-      default:
-        ensure_packagefiles_available(pkg);
-        ensure_diversions();
-        file= pkg->clientdata->files;
-        if (!file) {
-          printf(_("Package `%s' does not contain any files (!)\n"),pkg->name);
-        } else {
-          while (file) {
-            namenode= file->namenode;
-            puts(namenode->name);
-            if (namenode->divert && !namenode->divert->camefrom) {
-              if (!namenode->divert->pkg) printf(_("locally diverted"));
-              else if (pkg == namenode->divert->pkg) printf(_("package diverts others"));
-              else printf(_("diverted by %s"),namenode->divert->pkg->name);
-              printf(_(" to: %s\n"),namenode->divert->useinstead->name);
-            }
-            file= file->next;
-          }
-        }
-        break;
-      }
-      break;
-
-    default:
-      internerr("unknown action");
-    }
-        
-    putchar('\n');
-    if (ferror(stdout)) werr("stdout");
-  }
-
-  if (failures) {
-    nerrs++;
-    fputs(_("Use dpkg --info (= dpkg-deb --info) to examine archive files,\n"
-         "and dpkg --contents (= dpkg-deb --contents) to list their contents.\n"),stderr);
-    if (ferror(stdout)) werr("stdout");
-  }
-}
-
 static void assertversion(const char *const *argv,
 			struct versionrevision *verrev_buf,
 			const char *reqversion) {
@@ -729,7 +455,6 @@ void printarch(const char *const *argv) {
   if (fflush(stdout)) werr("stdout");
 }
 
-void cmpversions(const char *const *argv) NONRETURNING;
 void cmpversions(const char *const *argv) {
   struct relationinfo {
     const char *string;
