@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * help.c - various helper routines
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -91,7 +91,7 @@ void checkpath(void) {
     s= path;
     while (s) {
       p= strchr(s,':');
-      l= p ? p-s : strlen(s);
+      l= p ? p-s : (long)strlen(s);
       memcpy(buf,s,l);
       if (l) buf[l++]= '/';
       strcpy(buf+l,*clp);
@@ -135,7 +135,7 @@ void cu_closedir(int argc, void **argv) {
 }
 
 void cu_closefd(int argc, void **argv) {
-  int ip= *(int*)argv;
+  int ip= *(int*)argv[0];
   close(ip);
 }
 
