@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * main.c - main program
  *
- * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -47,7 +47,7 @@ static void printversion(void) {
   if (fputs( DPKG_VERSION_ARCH ".\n", stdout) < 0) werr("stdout");
   if (fputs(_( "This is free software; see the GNU General Public Licence version 2 or\n"
 		"later for copying conditions.  There is NO warranty.\n"
-		"See dpkg --licence for copyright and license details.\n"),
+		"See " DPKG " --licence for copyright and license details.\n"),
 		 stdout) < 0) werr("stdout");
 }
 /*
@@ -264,25 +264,26 @@ static void setinteger(const struct cmdinfo *cip, const char *value) {
   *cip->iassignto= v;
 }
 
-
-static void setstatuspipe(const struct cmdinfo *cip, const char *value) {
-  static struct pipef *lastpipe= NULL;
+static void setpipe(const struct cmdinfo *cip, const char *value) {
+  static struct pipef **lastpipe;
   unsigned long v;
   char *ep;
 
-  if (lastpipe) {
-    lastpipe->next= nfmalloc(sizeof(struct pipef));
-    lastpipe= lastpipe->next;
-  } else {
-    lastpipe= status_pipes= nfmalloc(sizeof(struct pipef));
-  }
   v= strtoul(value,&ep,0);
   if (*ep || v > INT_MAX)
     badusage(_("invalid integer for --%s: `%.250s'"),cip->olong,value);
-  lastpipe->fd= v;
+
+  lastpipe= cip->parg;
+  if (*lastpipe) {
+    (*lastpipe)->next= nfmalloc(sizeof(struct pipef));
+    *lastpipe= (*lastpipe)->next;
+  } else {
+    *lastpipe= nfmalloc(sizeof(struct pipef));
+  }
+  (*lastpipe)->fd= v;
 }
 
-void setforce(const struct cmdinfo *cip, const char *value) {
+static void setforce(const struct cmdinfo *cip, const char *value) {
   const char *comma;
   size_t l;
   const struct forceinfo *fip;
@@ -341,14 +342,9 @@ Forcing options marked [*] are enabled by default.\n"),
   }
 }
 
-static const char *const passlongopts[]= {
-  "build", "contents", "control", "info", "field", "extract", "new", "old",
-  "vextract", "fsys-tarfile", 0
-};
-
-static const char passshortopts[]= "bceIfxX";
 static const char okpassshortopts[]= "D";
 
+void execbackend(const char *const *argv) NONRETURNING;
 void commandfd(const char *const *argv);
 static const struct cmdinfo cmdinfos[]= {
   /* This table has both the action entries in it and the normal options.
@@ -359,6 +355,8 @@ static const struct cmdinfo cmdinfos[]= {
  { longopt, shortopt, 0,0,0, setaction, code, 0, (voidfnp)function }
 #define OBSOLETE(longopt,shortopt) \
  { longopt, shortopt, 0,0,0, setobsolete, 0, 0, 0 }
+#define ACTIONBACKEND(longopt,shortop, backend) \
+ { longopt, shortop, 0,0,0, setaction, 0, (void*)backend, (voidfnp)execbackend }
 
   ACTION( "install",                        'i', act_install,              archivefiles    ),
   ACTION( "unpack",                          0,  act_unpack,               archivefiles    ),
@@ -366,19 +364,19 @@ static const struct cmdinfo cmdinfos[]= {
   ACTION( "configure",                       0,  act_configure,            packages        ),
   ACTION( "remove",                         'r', act_remove,               packages        ),
   ACTION( "purge",                          'P', act_purge,                packages        ),
-  ACTION( "listfiles",                      'L', act_listfiles,     enqperpackage   ),
-  ACTION( "status",                         's', act_status,               enqperpackage   ),
+  ACTIONBACKEND( "listfiles",               'L', DPKGQUERY),
+  ACTIONBACKEND( "status",                  's', DPKGQUERY),
   ACTION( "get-selections",                  0,  act_getselections,        getselections   ),
   ACTION( "set-selections",                  0,  act_setselections,        setselections   ),
-  ACTION( "print-avail",                    'p', act_printavail,           enqperpackage   ),
+  ACTIONBACKEND( "print-avail",             'p', DPKGQUERY),
   ACTION( "update-avail",                    0,  act_avreplace,            updateavailable ),
   ACTION( "merge-avail",                     0,  act_avmerge,              updateavailable ),
   ACTION( "clear-avail",                     0,  act_avclear,              updateavailable ),
   ACTION( "forget-old-unavail",              0,  act_forgetold,            forgetold       ),
   ACTION( "audit",                          'C', act_audit,                audit           ),
   ACTION( "yet-to-unpack",                   0,  act_unpackchk,            unpackchk       ),
-  ACTION( "list",                           'l', act_listpackages,         listpackages    ),
-  ACTION( "search",                         'S', act_searchfiles,          searchfiles     ),
+  ACTIONBACKEND( "list",                    'l', DPKGQUERY),
+  ACTIONBACKEND( "search",                  'S', DPKGQUERY),
   ACTION( "print-architecture",              0,  act_printarch,            printarch       ),
   ACTION( "print-gnu-build-architecture",    0,  act_printgnuarch,         printarch       ),
   ACTION( "assert-support-predepends",       0,  act_assertpredep,         assertpredep    ),
@@ -392,7 +390,7 @@ static const struct cmdinfo cmdinfos[]= {
   ACTION( "command-fd",                   'c', act_commandfd,   commandfd     ),
 */
   
-  { "status-fd",	  0,   1,  0,              0,  setstatuspipe },
+  { "status-fd",	  0,   1,  0,              0,  setpipe, 0, &status_pipes },
   { "pending",           'a',  0,  &f_pending,     0,  0,             1              },
   { "recursive",         'R',  0,  &f_recursive,   0,  0,             1              },
   { "no-act",             0,   0,  &f_noact,       0,  0,             1              },
@@ -417,19 +415,40 @@ static const struct cmdinfo cmdinfos[]= {
   { "version",            0,   0,  0, 0,               versiononly                   },
   { "licence",/* UK spelling */ 0,0,0,0,               showcopyright                 },
   { "license",/* US spelling */ 0,0,0,0,               showcopyright                 },
+  ACTIONBACKEND( "build",		'b', BACKEND),
+  ACTIONBACKEND( "contents",		'c', BACKEND),
+  ACTIONBACKEND( "control",		'e', BACKEND),
+  ACTIONBACKEND( "info",		'I', BACKEND),
+  ACTIONBACKEND( "field",		'f', BACKEND),
+  ACTIONBACKEND( "extract",		'x', BACKEND),
+  ACTIONBACKEND( "new",			0,  BACKEND),
+  ACTIONBACKEND( "old",			0,  BACKEND),
+  ACTIONBACKEND( "vextract",		'X', BACKEND),
+  ACTIONBACKEND( "fsys-tarfile",	0,   BACKEND),
   {  0,                   0                                                          }
 };
 
-static void execbackend(int argc, const char *const *argv) NONRETURNING;
-static void execbackend(int argc, const char *const *argv) {
-  char **nargv= malloc(sizeof(char *) * argc + 1);
-  int i;
+void execbackend(const char *const *argv) {
+  char **nargv;
+  int i, argc = 1;
+  const char *const *arg = argv;
+  while(*arg != 0) { arg++; argc++; }
+  nargv= malloc(sizeof(char *) * (argc + 2));
+
   if (!nargv) ohshite(_("couldn't malloc in execbackend"));
-  for (i= 0; i < argc; i++)
-    nargv[i]= strdup(argv[i]);
+  nargv[0]= strdup(cipaction->parg);
+  if (!nargv[0]) ohshite(_("couldn't strdup in execbackend"));
+  nargv[1]= malloc(strlen(cipaction->olong) + 3);
+  if (!nargv[1]) ohshite(_("couldn't malloc in execbackend"));
+  strcpy(nargv[1], "--");
+  strcat(nargv[1], cipaction->olong);
+  for (i= 2; i <= argc; i++) {
+    nargv[i]= strdup(argv[i-2]);
+    if (!nargv[i]) ohshite(_("couldn't strdup in execbackend"));
+  }
   nargv[i]= 0;
-  execvp(BACKEND, nargv);
-  ohshite(_("failed to exec dpkg-deb"));
+  execvp(cipaction->parg, nargv);
+  ohshite(_("failed to exec %s"),(char *)cipaction->parg);
 }
 void commandfd(const char *const *argv) {
   jmp_buf ejbuf;
@@ -527,38 +546,9 @@ printf("line=`%*s'\n",(int)linevb.used,linevb.buf);
 
 int main(int argc, const char *const *argv) {
   jmp_buf ejbuf;
-  int c;
-  const char *argp, *const *blongopts, *const *argvs;
   static void (*actionfunction)(const char *const *argv);
 
-  setlocale(LC_ALL, "");
-  bindtextdomain(PACKAGE, LOCALEDIR);
-  textdomain(PACKAGE);
-
-  if (setjmp(ejbuf)) { /* expect warning about possible clobbering of argv */
-    error_unwind(ehflag_bombout); exit(2);
-  }
-  push_error_handler(&ejbuf,print_error_fatal,0);
-
-  umask(022); /* Make sure all our status databases are readable. */
- 
-  for (argvs=argv+1; (argp= *argvs) && *argp++=='-'; argvs++) {
-    if (*argp++=='-') {
-      if (!strcmp(argp,"-")) break;
-      for (blongopts=passlongopts; *blongopts; blongopts++) {
-        if (!strcmp(argp,*blongopts)) execbackend(argc,argv);
-      }
-    } else {
-      if (!*--argp) break;
-      while ((c= *argp++)) {
-        if (strchr(passshortopts,c)) execbackend(argc,argv);
-        if (!strchr(okpassshortopts,c)) break;
-      }
-    }
-  }
-
-  myfileopt(CONFIGDIR "/" DPKG ".cfg", cmdinfos);
-  myopt(&argv,cmdinfos);
+  standard_startup(&ejbuf, argc, &argv, DPKG, 1, cmdinfos);
   if (!cipaction) badusage(_("need an action option"));
 
   setvbuf(stdout,0,_IONBF,0);
@@ -568,8 +558,7 @@ int main(int argc, const char *const *argv) {
 
   actionfunction(argv);
 
-  set_error_display(0,0);
-  error_unwind(ehflag_normaltidy);
+  standard_shutdown(0);
 
   return reportbroken_retexitstatus();
 }
