@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * processarc.c - the huge function process_archive
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -191,7 +191,7 @@ void process_archive(const char *filename) {
   strcpy(cidirrest,CONTROLFILE);
 
   parsedb(cidir, pdb_recordavailable|pdb_rejectstatus|pdb_ignorefiles|pdb_weakclassification,
-          &pkg,0,0);
+          &pkg,NULL,NULL);
   if (!pkg->files) {
     pkg->files= nfmalloc(sizeof(struct filedetails));
     pkg->files->next= 0;
@@ -217,6 +217,12 @@ void process_archive(const char *filename) {
   if (!pkg->installed.valid) blankpackageperfile(&pkg->installed);
   assert(pkg->available.valid);
 
+  for (deconpil= deconfigure;
+       deconpil;
+       deconpil= deconpiltemp) {
+    deconpiltemp= deconpil->next;
+    free(deconpil);
+  }
   deconfigure= 0;
   clear_istobes();
 
@@ -787,6 +793,8 @@ void process_archive(const char *filename) {
   pkg->installed.architecture= 0; /* This is irrelevant in the status file. */
   pkg->installed.installedsize= pkg->available.installedsize;
   pkg->installed.version= pkg->available.version;
+  pkg->installed.origin = pkg->available.origin;                               
+  pkg->installed.bugs = pkg->available.bugs;                                   
 
   /* We have to generate our own conffiles structure. */
   pkg->installed.conffiles= 0; iconffileslastp= &pkg->installed.conffiles;
