@@ -47,23 +47,29 @@ $dpkglibdir="/usr/lib/dpkg";
 push(@INC,$dpkglibdir);
 require 'controllib.pl';
 
-%archtable=('i386',      'i386-linux',
-	    'sparc',     'sparc-linux',
-	    'sparc64',   'sparc64-linux',
-	    'alpha',     'alpha-linux',
-	    'm68k',      'm68k-linux',
-            'arm',       'arm-linux',
-            'powerpc',   'powerpc-linux',
-	    'mips',      'mips-linux',
-	    'mipsel',    'mipsel-linux',
-	    'sh',        'sh-linux',
-	    'sheb',      'sheb-linux',
-	    'hppa',      'hppa-linux',
-	    'hurd-i386', 'i386-gnu',
-	    's390',	 's390-linux',
-	    'ia64',	 'ia64-linux',
-	    'openbsd-i386', 'i386-openbsd',
-	    'freebsd-i386', 'i386-freebsd');
+%archtable=('i386',		'i386-linux',
+	    'sparc',		'sparc-linux',
+	    'sparc64',		'sparc64-linux',
+	    'alpha',		'alpha-linux',
+	    'm68k',		'm68k-linux',
+            'arm',		'arm-linux',
+            'powerpc',		'powerpc-linux',
+	    'mips',		'mips-linux',
+	    'mipsel',		'mipsel-linux',
+	    'sh3',		'sh3-linux',
+	    'sh4',		'sh4-linux',
+	    'sh3eb',		'sh3eb-linux',
+	    'sh4eb',		'sh4eb-linux',
+	    'hppa',		'hppa-linux',
+	    'hurd-i386',	'i386-gnu',
+	    's390',		's390-linux',
+	    's390x',		's390x-linux',
+	    'ia64',		'ia64-linux',
+	    'openbsd-i386',	'i386-openbsd',
+	    'freebsd-i386',	'i386-freebsd',
+	    'netbsd-i386',	'i386-netbsdelf-gnu',
+	    'darwin-powerpc',	'powerpc-darwin',
+	    'darwin-i386',	'i386-darwin');
 
 sub usageversion {
     print STDERR
@@ -94,7 +100,6 @@ sub rewrite_gnu_cpu {
 
 	s/(?:i386|i486|i586|i686|pentium)(.*linux)/i386$1/;
 	s/ppc/powerpc/;
-	s/sh[34]/sh/;
 	return $_;
 }
 
@@ -119,10 +124,9 @@ if ($?>>8) {
 }
 chomp $deb_build_arch;
 $deb_build_gnu_type = $archtable{$deb_build_arch};
-$deb_build_gnu_cpu = $deb_build_gnu_type;
-$deb_build_gnu_system = $deb_build_gnu_type;
-$deb_build_gnu_cpu =~ s/-.*$//;
-$deb_build_gnu_system =~ s/^.*-//;
+@deb_build_gnu_triple = split(/-/, $deb_build_gnu_type, 2);
+$deb_build_gnu_cpu = $deb_build_gnu_triple[0];
+$deb_build_gnu_system = $deb_build_gnu_triple[1];
 
 # Default host: Current gcc.
 $gcc = `\${CC:-gcc} --print-libgcc-file-name`;
@@ -149,8 +153,9 @@ if ($gcc ne '') {
 	$gcc=$archtable{$list[0]};
 	$deb_host_arch = $list[0];
 	$deb_host_gnu_type = $gcc;
-        ($deb_host_gnu_system = $gcc) =~ s/^.*-//;
-        ($deb_host_gnu_cpu = $gcc ) =~ s/-.*$//;
+	@deb_host_gnu_triple = split(/-/, $deb_host_gnu_type, 2);
+	$deb_host_gnu_cpu = $deb_host_gnu_triple[0];
+	$deb_host_gnu_system = $deb_host_gnu_triple[1];
     }
 }
 if (!defined($deb_host_arch)) {
@@ -209,9 +214,10 @@ die "couldn't parse GNU system type $req_host_gnu_type, must be arch-os or arch-
 
 $deb_host_arch = $req_host_arch if $req_host_arch ne '';
 if ($req_host_gnu_type ne '') {
-    $deb_host_gnu_cpu = $deb_host_gnu_system = $deb_host_gnu_type = $req_host_gnu_type;
-    $deb_host_gnu_cpu =~ s/-.*$//;
-    $deb_host_gnu_system =~ s/^.*-//;
+    $deb_host_gnu_type = $req_host_gnu_type;
+    @deb_host_gnu_triple = split(/-/, $deb_host_gnu_type, 2);
+    $deb_host_gnu_cpu = $deb_host_gnu_triple[0];
+    $deb_host_gnu_system = $deb_host_gnu_triple[1];
 }
 
 #$gcc = `\${CC:-gcc} --print-libgcc-file-name`;
@@ -320,11 +326,11 @@ some other script (for example dpkg-buildpackage).
 
 =item build machine
 
-The machine the package is build on.
+The machine the package is built on.
 
 =item host machine
 
-The machine the package is build for.
+The machine the package is built for.
 
 =item Debian Architecture
 
@@ -390,9 +396,12 @@ The System part of DEB_HOST_GNU_TYPE
 =head1 DEBIAN/RULES
 
 The environment variables set by dpkg-architecture are passed to
-debian/rules as make variables (see make documentation). You can and should
-use them in the build process as needed. Here are some examples, which also
-show how you can improve the cross compilation support in your package:
+debian/rules as make variables (see make documentation).  However, you
+should not rely on them, as this breaks manual invocation of the
+script.  Instead, you should always initialize them using
+dpkg-architecture with the -q option.  Here are some examples, which
+also show how you can improve the cross compilation support in your
+package:
 
 Instead:
 
@@ -401,9 +410,10 @@ configure $(ARCH)-linux
 
 please use the following:
 
-B_ARCH=$(DEB_BUILD_GNU_TYPE)
-H_ARCH=$(DEB_HOST_GNU_TYPE)
-configure --build=$(B_ARCH) --host=$(H_ARCH)
+DEB_BUILD_GNU_TYPE := $(shell dpkg-architecture -qDEB_BUILD_GNU_TYPE)
+DEB_HOST_GNU_TYPE := $(shell dpkg-architecture -qDEB_HOST_GNU_TYPE)
+
+configure --build=$(DEB_BUILD_GNU_TYPE) --host=$(DEB_BUILD_GNU_SYSTEM)
 
 Instead:
 
@@ -414,8 +424,9 @@ endif
 
 please use:
 
-ARCH=$(DEB_HOST_ARCH)
-ifeq ($(ARCH),alpha)
+DEB_HOST_ARCH := $(shell dpkg-architecture -qDEB_HOST_ARCH)
+
+ifeq ($(DEB_HOST_ARCH),alpha)
   ...
 endif
 
@@ -426,10 +437,11 @@ Debian architectures which don't equal a processor name.
 
 =head1 BACKWARD COMPATIBILITY
 
-When providing a new facility, it is always a good idea to stay compatible with old
-versions of the programs. Note that dpkg-architecture does not affect old
-debian/rules files, so the only thing to consider is using old building
-scripts with new debian/rules files. The following does the job:
+When providing a new facility, it is always a good idea to stay
+compatible with old versions of the programs.  Note that
+dpkg-architecture does not affect old debian/rules files, so the only
+thing to consider is using old versions of dpkg-dev with new
+debian/rules files.  The following does the job:
 
 DEB_BUILD_ARCH := $(shell dpkg --print-installation-architecture)
 DEB_BUILD_GNU_CPU := $(patsubst hurd-%,%,$(DEB_BUILD_ARCH))
@@ -440,10 +452,10 @@ else
 endif
 DEB_BUILD_GNU_TYPE=$(DEB_BUILD_GNU_CPU)-$(DEB_BUILD_GNU_SYSTEM)
 
-DEB_HOST_ARCH=$(DEB_BUILD_ARCH)
-DEB_HOST_GNU_CPU=$(DEB_BUILD_GNU_CPU)
-DEB_HOST_GNU_SYSTEM=$(DEB_BUILD_GNU_SYSTEM)
-DEB_HOST_GNU_TYPE=$(DEB_BUILD_GNU_TYPE)
+DEB_HOST_ARCH := $(DEB_BUILD_ARCH)
+DEB_HOST_GNU_CPU := $(DEB_BUILD_GNU_CPU)
+DEB_HOST_GNU_SYSTEM := $(DEB_BUILD_GNU_SYSTEM)
+DEB_HOST_GNU_TYPE := $(DEB_BUILD_GNU_TYPE)
 
 Put a subset of these lines at the top of your debian/rules file; these
 default values will be overwritten if dpkg-architecture is used.
@@ -455,20 +467,6 @@ is sufficient (this is indeed the Debian architecture of the build machine,
 but remember that we are only trying to be backward compatible with native
 compilation).
 
-You may not want to care about old build packages (for example, if you have
-sufficient source dependencies declared anyway). But you should at least
-support the traditional way to build packages by calling `debian/rules
-build' directly, without setting environment variables. To do this, use the
-B<-q> option to query suitable default values:
-
-DEB_BUILD_ARCH=`dpkg-architecture -qDEB_BUILD_ARCH`
-DEB_BUILD_GNU=`dpkg-architecture -qDEB_BUILD_GNU`
-
-etc. You get the idea. This way, you can ensure that the variables are never
-undeclared. Note that this breaks backwards compatibility with old build
-scripts, and you should only do that if source dependencies are implemented
-and declared accordingly.
-
 =head1 SEE ALSO
 
 dpkg-buildpackage
