@@ -8,6 +8,7 @@ $version= '1.2.6'; # This line modified by Makefile
         'package_revision','revision');
 
 @fieldpri= ('Package',
+	    'Source',
             'Version',
             'Priority',
             'Section',
@@ -23,6 +24,7 @@ $version= '1.2.6'; # This line modified by Makefile
             'Architecture',
             'Filename',
             'Size',
+	    'Installed-Size',
             'MD5sum',
             'Description');
 
@@ -102,8 +104,8 @@ while (<F>) {
 
     open(C,"md5sum <$fn |") || die "$fn $!";
     chop($_=<C>); close(C); $? and die "\`md5sum < $fn' exited with $?\n";
-    /^[0-9a-f]{32}$/ or die "Strange text from \`md5sum < $fn': \`$_'\n";
-    $tv{'MD5sum'}= $_;
+    /^([0-9a-f]{32})\s*-?\s*$/ or die "Strange text from \`md5sum < $fn': \`$_'\n";
+    $tv{'MD5sum'}= $1;
 
     @stat= stat($fn) or die "Couldn't stat $fn: $!\n";
     $stat[7] or die "$fn is empty\n";
