@@ -13,14 +13,15 @@
  *   to support GNU gettext.
  * This file is in the public domain.
  */
+
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <getopt.h>
 #include <unistd.h>
 
-#include "config.h"
-
 /* Take care of NLS matters.  */
 
 #if HAVE_LOCALE_H
@@ -56,17 +57,16 @@
 #endif
 #endif
 
-extern char *optarg;
-extern int optind;
 const char printforhelp[]= N_("Type md5sum --help for help.");
 const char thisname[]= MD5SUM;
 
-void usage(void);
+void usage(void) NONRETURNING;
 void print_digest(unsigned char *p);
 int mdfile(int fd, unsigned char **digest);
 int do_check(FILE *chkf);
 int hex_digit(int c);
 int get_md5_line(FILE *fp, unsigned char *digest, char *file);
+int process_arg(const char* arg, int bin_mode, unsigned char **digest);
 
 char *progname;
 int verbose = 0;
@@ -78,7 +78,7 @@ print_md5sum_error(const char* emsg, const char* arg) {
 	fprintf(stderr, _("error processing %s: %s\n"), arg, emsg);
 }
 
-void cu_closefile(int argc, void** argv) {
+static void cu_closefile(int argc, void** argv) {
 	FILE *f = (FILE*)(argv[0]);
 	fclose(f);
 }
@@ -127,44 +127,61 @@ main(int argc, char **argv)
 		push_error_handler(&ejbuf, print_md5sum_error, "stdin");
 
 		mdfile(fileno(stdin), &digest);
-		printf("%s\n", digest);
+		printf("%s %c-\n", digest, bin_mode ? '*' : ' ');
 		set_error_display(0, 0);
 		error_unwind(ehflag_normaltidy);
 		exit(0);
 	}
 	for ( ; argc > 0; --argc, ++argv) {
-		if (setjmp(ejbuf)) {
-			error_unwind(ehflag_bombout);
-			rc++;
-			continue;
+		switch (process_arg(*argv, bin_mode, &digest)) {
+			case 1:
+				rc++;
+				break;
+			case 2:
+				perror(*argv);
+				rc++;
+				break;
+			case 3:
+				perror(*argv);
+				rc++;
+				break;
+			default:
+				printf("%s %c%s\n", digest, bin_mode ? '*' : ' ', *argv);
 		}
-		push_error_handler(&ejbuf, print_md5sum_error, *argv);
-		if (bin_mode)
-			fp = fopen(*argv, FOPRBIN);
-		else
-			fp = fopen(*argv, FOPRTXT);
-		if (fp == NULL) {
-			perror(*argv);
-			rc++;
-			continue;
-		}
-		push_cleanup(cu_closefile,ehflag_bombout, 0,0, 1,(void*)fp);
-		
-		mdfile(fileno(fp), &digest);
-		printf("%s %c%s\n", digest, bin_mode ? '*' : ' ', *argv);
-		pop_cleanup(ehflag_normaltidy); /* fd= fopen() */
-		fclose(fp);
-		fp= NULL;
-		set_error_display(0, 0);
-		error_unwind(ehflag_normaltidy);
 	}
-
 	return rc;
 }
 
-void usage() NONRETURNING;
+int process_arg(const char* arg, int bin_mode, unsigned char **digest)
+{
+	jmp_buf ejbuf;
+	FILE *fp = NULL;
+
+	if (setjmp(ejbuf)) {
+		error_unwind(ehflag_bombout);
+		return 1;
+	}
+	push_error_handler(&ejbuf, print_md5sum_error, arg);
+	if (bin_mode)
+		fp = fopen(arg, FOPRBIN);
+	else
+		fp = fopen(arg, FOPRTXT);
+	if (fp == NULL)
+		return 2;
+	push_cleanup(cu_closefile,ehflag_bombout, 0,0, 1,(void*)fp);
+	if (mdfile(fileno(fp), digest)) {
+		fclose(fp);
+		return 3;
+	}
+	pop_cleanup(ehflag_normaltidy); /* fd= fopen() */
+	fclose(fp);
+	set_error_display(0, 0);
+	error_unwind(ehflag_normaltidy);
+	return 0;
+}
+
 void
-usage()
+usage(void)
 {
         fputs(_("usage: md5sum [-bv] [-c [file]] | [file...]\n\
 Generates or checks MD5 Message Digests\n\
@@ -237,7 +254,6 @@ do_check(FILE *chkf)
 	int rc, ex = 0, failed = 0, checked = 0;
 	unsigned char chk_digest[32], *file_digest = NULL;
 	char filename[256];
-	FILE *fp;
 	size_t flen = 14;
 
 	while ((rc = get_md5_line(chkf, chk_digest, filename)) >= 0) {
@@ -248,22 +264,16 @@ do_check(FILE *chkf)
 				flen = strlen(filename);
 			fprintf(stderr, "%-*s ", (int)flen, filename);
 		}
-		if (bin_mode || rc == 2)
-			fp = fopen(filename, FOPRBIN);
-		else
-			fp = fopen(filename, FOPRTXT);
-		if (fp == NULL) {
-			fprintf(stderr, _("%s: can't open %s\n"), progname, filename);
-			ex = 2;
-			continue;
+		switch (process_arg(filename, bin_mode || rc == 2, &file_digest)) {
+			case 2:
+				fprintf(stderr, _("%s: can't open %s\n"), progname, filename);
+				ex = 2;
+				continue;
+			case 3:
+				fprintf(stderr, _("%s: error reading %s\n"), progname, filename);
+				ex = 2;
+				continue;
 		}
-		if (mdfile(fileno(fp), &file_digest)) {
-			fprintf(stderr, _("%s: error reading %s\n"), progname, filename);
-			ex = 2;
-			fclose(fp);
-			continue;
-		}
-		fclose(fp);
 		if (memcmp(chk_digest, file_digest, 32) != 0) {
 			if (verbose)
 				fprintf(stderr, _("FAILED\n"));
