@@ -30,6 +30,12 @@
 #  define OSsunos
 #elif defined(OPENBSD)
 #  define OSOpenBSD
+#elif defined(hpux)
+#  define OShpux
+#elif defined(__FreeBSD__)
+#  define OSFreeBSD
+#elif defined(__NetBSD__)
+#  define OSNetBSD
 #else
 #  error Unknown architecture - cannot build start-stop-daemon
 #endif
@@ -41,7 +47,7 @@
 #  include <ps.h>
 #endif
 
-#if defined(OSOpenBSD)
+#if defined(OSOpenBSD) || defined(OSFreeBSD) || defined(OSNetBSD)
 #include <sys/param.h>
 #include <sys/user.h>
 #include <sys/proc.h>
@@ -54,6 +60,11 @@
 #include <limits.h>
 #endif
 
+#if defined(OShpux)
+#include <sys/param.h>
+#include <sys/pstat.h>
+#endif
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -140,16 +151,16 @@ static void check(pid_t pid);
 static void do_pidfile(const char *name);
 static void do_stop(int signal_nr, int quietmode,
 		    int *n_killed, int *n_notkilled, int retry_nr);
-#if defined(OSLinux)
+#if defined(OSLinux) || defined(OShpux)
 static int pid_is_exec(pid_t pid, const struct stat *esb);
 #endif
 
 
 #ifdef __GNUC__
 static void fatal(const char *format, ...)
-	__attribute__((noreturn, format(printf, 1, 2)));
+	NONRETURNPRINTFFORMAT(1, 2);
 static void badusage(const char *msg)
-	__attribute__((noreturn));
+	NONRETURNING;
 #else
 static void fatal(const char *format, ...);
 static void badusage(const char *msg);
@@ -321,7 +332,7 @@ const struct sigpair siglist[] = {
 	{ "TTOU",	SIGTTOU	}
 };
 
-static int parse_integer (const char *string, int *value_r) {
+static int parse_integer(const char *string, int *value_r) {
 	unsigned long ul;
 	char *ep;
 
@@ -336,7 +347,7 @@ static int parse_integer (const char *string, int *value_r) {
 	return 0;
 }
 
-static int parse_signal (const char *signal_str, int *signal_nr)
+static int parse_signal(const char *signal_str, int *signal_nr)
 {
 	unsigned int i;
 
@@ -535,7 +546,7 @@ parse_options(int argc, char * const *argv)
 	if (signal_str != NULL) {
 		if (parse_signal (signal_str, &signal_nr) != 0)
 			badusage("signal value must be numeric or name"
-				 " of signal (KILL, INTR, ...)");
+				 " of signal (KILL, INT, ...)");
 	}
 
 	if (schedule_str != NULL) {
@@ -647,13 +658,28 @@ pid_is_cmd(pid_t pid, const char *name)
 #endif /* OSHURD */
 
 
+static int
+pid_is_running(pid_t pid)
+{
+	struct stat sb;
+	char buf[32];
+
+	sprintf(buf, "/proc/%d", pid);
+	if (stat(buf, &sb) != 0) {
+		if (errno!=ENOENT)
+			fatal("Error stating %s: %s", buf, strerror(errno));
+		return 0;
+	}
+
+	return 1;
+}
 
 static void
 check(pid_t pid)
 {
-#if defined(OSLinux)
+#if defined(OSLinux) || defined(OShpux)
 	if (execname && !pid_is_exec(pid, &exec_stat))
-#elif defined(OSHURD)
+#elif defined(OSHURD) || defined(OSFreeBSD) || defined(OSNetBSD)
     /* I will try this to see if it works */
 	if (execname && !pid_is_cmd(pid, execname))
 #endif
@@ -662,6 +688,8 @@ check(pid_t pid)
 		return;
 	if (cmdname && !pid_is_cmd(pid, cmdname))
 		return;
+	if (start && !pid_is_running(pid))
+		return;
 	push(&found, pid);
 }
 
@@ -683,7 +711,8 @@ do_pidfile(const char *name)
 
 /* WTA: this  needs to be an autoconf check for /proc/pid existance.
  */
-#if defined(OSLinux) || defined (OSsunos)
+
+#if defined(OSLinux) || defined (OSsunos) || defined(OSfreebsd)
 static void
 do_procinit(void)
 {
@@ -712,12 +741,12 @@ do_procinit(void)
 
 #if defined(OSHURD)
 error_t
-check_all (void *ptr)
+check_all(void *ptr)
 {
 	struct proc_stat *pstat = ptr;
 
-	check (pstat->pid);
-	return (0);
+	check(pstat->pid);
+	return 0;
 }
 
 static void
@@ -726,26 +755,26 @@ do_procinit(void)
 	struct ps_context *context;
 	error_t err;
 
-	err = ps_context_create (getproc (), &context);
+	err = ps_context_create(getproc(), &context);
 	if (err)
-		error (1, err, "ps_context_create");
+		error(1, err, "ps_context_create");
 
-	err = proc_stat_list_create (context, &procset);
+	err = proc_stat_list_create(context, &procset);
 	if (err)
-		error (1, err, "proc_stat_list_create");
+		error(1, err, "proc_stat_list_create");
 
-	err = proc_stat_list_add_all (procset, 0, 0);
+	err = proc_stat_list_add_all(procset, 0, 0);
 	if (err)
-		error (1, err, "proc_stat_list_add_all");
+		error(1, err, "proc_stat_list_add_all");
 
 	/* Check all pids */
-	ihash_iterate (context->procs, check_all);
+	ihash_iterate(context->procs, check_all);
 }
 #endif /* OSHURD */
 
 
-#if defined(OSOpenBSD)
-int
+#if defined(OSOpenBSD) || defined(OSFreeBSD) || defined(OSNetBSD)
+static int
 pid_is_cmd(pid_t pid, const char *name)
 {
         kvm_t *kd;
@@ -761,14 +790,14 @@ pid_is_cmd(pid_t pid, const char *name)
                 errx(1, "%s", errbuf);
         if ((kp = kvm_getprocs(kd, KERN_PROC_PID, pid, &nentries)) == 0)
                 errx(1, "%s", kvm_geterr(kd));
-	if ( ( pid_argv_p = kvm_getargv(kd, kp, argv_len)) == 0)
+	if ((pid_argv_p = kvm_getargv(kd, kp, argv_len)) == 0)
 		errx(1, "%s", kvm_geterr(kd)); 
 
 	start_argv_0_p = *pid_argv_p;
 	/* find and compare string */
 	  
 	/* find end of argv[0] then copy and cut of str there. */
-	if ( (end_argv_0_p = strchr(*pid_argv_p, ' ')) == 0 ) 	
+	if ((end_argv_0_p = strchr(*pid_argv_p, ' ')) == 0 ) 	
 	/* There seems to be no space, so we have the command
 	 * allready in its desired form. */
 	  start_argv_0_p = *pid_argv_p;
@@ -786,8 +815,8 @@ pid_is_cmd(pid_t pid, const char *name)
         return (strcmp(name, start_argv_0_p) == 0) ? 1 : 0;
 }
  
-int
-pid_is_user(pid_t pid, int uid)
+static int
+pid_is_user(pid_t pid, uid_t uid)
 {
 	kvm_t *kd;
 	int nentries;   /* Value not used */
@@ -801,7 +830,7 @@ pid_is_user(pid_t pid, int uid)
 		errx(1, "%s", errbuf);
 	if ((kp = kvm_getprocs(kd, KERN_PROC_PID, pid, &nentries)) == 0)
 		errx(1, "%s", kvm_geterr(kd));
-	if ( kp->kp_proc.p_cred )
+	if (kp->kp_proc.p_cred )
 		kvm_read(kd, (u_long)&(kp->kp_proc.p_cred->p_ruid),
 			&proc_uid, sizeof(uid_t));
 	else
@@ -809,7 +838,7 @@ pid_is_user(pid_t pid, int uid)
 	return (proc_uid == (uid_t)uid);
 }
 
-int
+static int
 pid_is_exec(pid_t pid, const char *name)
 {
 	kvm_t *kd;
@@ -838,6 +867,54 @@ do_procinit(void)
 #endif /* OSOpenBSD */
 
 
+#if defined(OShpux)
+static int
+pid_is_user(pid_t pid, uid_t uid)
+{
+	struct pst_status pst;
+
+	if (pstat_getproc(&pst, sizeof(pst), (size_t) 0, (int) pid) < 0)
+		return 0;
+	return ((uid_t) pst.pst_uid == uid);
+}
+
+static int
+pid_is_cmd(pid_t pid, const char *name)
+{
+	struct pst_status pst;
+
+	if (pstat_getproc(&pst, sizeof(pst), (size_t) 0, (int) pid) < 0)
+		return 0;
+	return (strcmp(pst.pst_ucomm, name) == 0);
+}
+
+static int
+pid_is_exec(pid_t pid, const struct stat *esb)
+{
+	struct pst_status pst;
+
+	if (pstat_getproc(&pst, sizeof(pst), (size_t) 0, (int) pid) < 0)
+		return 0;
+	return ((dev_t) pst.pst_text.psf_fsid.psfs_id == esb->st_dev
+		&& (ino_t) pst.pst_text.psf_fileid == esb->st_ino);
+}
+
+static void
+do_procinit(void)
+{
+	struct pst_status pst[10];
+	int i, count;
+	int idx = 0;
+
+	while ((count = pstat_getproc(pst, sizeof(pst[0]), 10, idx)) > 0) {
+                for (i = 0; i < count; i++)
+			check(pst[i].pst_pid);
+                idx = pst[count - 1].pst_idx + 1;
+	}
+}
+#endif /* OShpux */
+
+
 static void
 do_findprocs(void)
 {
@@ -866,10 +943,11 @@ do_stop(int signal_nr, int quietmode, int *n_killed, int *n_notkilled, int retry
  	clear(&killed);
 
 	for (p = found; p; p = p->next) {
-		if (testmode)
+		if (testmode) {
 			printf("Would send signal %d to %d.\n",
 			       signal_nr, p->pid);
- 		else if (kill(p->pid, signal_nr) == 0) {
+			(*n_killed)++;
+		} else if (kill(p->pid, signal_nr) == 0) {
 			push(&killed, p->pid);
  			(*n_killed)++;
 		} else {
@@ -1137,21 +1215,37 @@ main(int argc, char **argv)
 			exit(0);
 		}
 		 /* child continues here */
+
+		/* create a new session */
+#ifdef HAVE_SETSID
+		setsid();
+#else
+		setpgid(0,0);
+#endif
+
+#if defined(OShpux)
+		 /* now close all extra fds */
+		for (i=sysconf(_SC_OPEN_MAX)-1; i>=0; --i) close(i);
+#else
 		 /* now close all extra fds */
 		for (i=getdtablesize()-1; i>=0; --i) close(i);
+#endif
+
+#ifdef HAVE_TIOCNOTTY
 		 /* change tty */
 		fd = open("/dev/tty", O_RDWR);
 		ioctl(fd, TIOCNOTTY, 0);
 		close(fd);
+#endif
 		chdir("/");
 		umask(022); /* set a default for dumb programs */
-		setpgid(0,0);  /* set the process group */
 		fd=open("/dev/null", O_RDWR); /* stdin */
 		dup(fd); /* stdout */
 		dup(fd); /* stderr */
 	}
 	if (nicelevel) {
-		if (nice(nicelevel))
+		errno=0;
+		if ((nice(nicelevel)==-1) && (errno!=0))
 			fatal("Unable to alter nice level by %i: %s", nicelevel,
 				strerror(errno));
 	}
